import * as defs from '../../definitions';
/**
 * Intersections are a bit complicated due to the way that additionalProperties works in JSON-Schema. To
 * support this we need to do different operations based on the type of codecs contained by the
 * intersection.
 *
 * If the intersection contains only object schemas then they are all merged under a single schema so
 * that `additionalProperties` applies against them simultaneously.
 *
 * If the intersection contains a mix of object schemas and unions then we need to construct the cartesian
 * product of all unions against all object schemas, combining them under a single `anyOf` definition. Each
 * product will be a merged schema with `additionalProperties` applying against all properties contained
 * within.
 *
 * Note: You can only really intersect object schemas and so we do not check for any other type of schema
 */
export declare const IntersectionParser: import("..").Parser<string, defs.Intersection<defs.AnyCodec, defs.AnyCodec>>;
export declare const UnionParser: import("..").Parser<string, defs.Union<defs.AnyCodec, defs.AnyCodec>>;
